"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStreamRouter = void 0;
const pump_1 = __importDefault(require("pump"));
const range_parser_1 = __importDefault(require("range-parser"));
const express_1 = require("express");
const http_errors_1 = require("http-errors");
const querystring_1 = require("querystring");
const torrent_client_1 = require("../services/torrent-client");
const helpers_1 = require("../helpers");
const validation_1 = require("../helpers/validation");
function getStreamRouter(config, _logger, client) {
    const encodeToken = config.security.streamApi.key || config.security.apiKey;
    const parseParams = (params) => {
        if (encodeToken) {
            if (params.file || params.fileIndex || params.fileType) {
                throw new http_errors_1.BadRequest(`All parameters should be encoded with JWT`);
            }
            const data = helpers_1.verifyJwtToken(validation_1.validateString(params.token || params.torrent, 'torrent'), encodeToken, config.security.streamApi.maxAge);
            if (!data) {
                throw new http_errors_1.Forbidden('Incorrect JWT encoding');
            }
            return data;
        }
        return {
            torrent: validation_1.validateString(params.torrent, 'torrent'),
            file: params.file ? validation_1.validateString(params.file, 'file') : undefined,
            fileType: params.fileType ? validation_1.validateString(params.fileType, 'fileType') : undefined,
            fileIndex: params.fileIndex ? validation_1.validateInt(params.fileIndex, 'fileIndex') : undefined,
        };
    };
    return express_1.Router()
        .get('/stream/:torrent(*)', async (req, res) => {
        const { torrent: link, ...params } = parseParams({
            torrent: req.params.torrent,
            ...req.query,
        });
        const torrent = await client.addTorrent(link);
        const file = torrent_client_1.filterFiles(torrent.files, params)[0];
        if (!file) {
            throw new http_errors_1.NotFound();
        }
        res.setHeader('Accept-Ranges', 'bytes');
        res.attachment(file.name);
        req.connection.setTimeout(3600000);
        const parsedRange = req.headers.range
            ? range_parser_1.default(file.length, req.headers.range)
            : undefined;
        const range = parsedRange instanceof Array ? parsedRange[0] : undefined;
        if (range) {
            res.statusCode = 206;
            res.setHeader('Content-Length', range.end - range.start + 1);
            res.setHeader('Content-Range', 'bytes ' + range.start + '-' + range.end + '/' + file.length);
        }
        else {
            res.setHeader('Content-Length', file.length);
        }
        if (req.method === 'HEAD') {
            return res.end();
        }
        return pump_1.default(file.createReadStream(range), res, () => {
            file.stop();
        });
    })
        .get('/stream', async (req, res) => {
        const { torrent, ...params } = parseParams(req.query);
        return res.redirect(`/stream/${encodeURIComponent(torrent)}?${querystring_1.stringify(params)}`, 301);
    })
        .get('/playlist/:torrent(*)', async (req, res) => {
        const domain = req.protocol + '://' + req.get('host');
        const { torrent: link, ...params } = parseParams({
            torrent: req.params.torrent,
            ...req.query,
        });
        const torrent = await client.addTorrent(link);
        const files = torrent_client_1.filterFiles(torrent.files, params).filter((v) => v.type.includes('video') || v.type.includes('audio'));
        req.connection.setTimeout(10000);
        res.attachment(torrent.name + `.m3u`);
        res.send([
            '#EXTM3U',
            ...files.flatMap((f) => [
                `#EXTINF:-1,${f.name}`,
                `${domain}${helpers_1.getSteamUrl(link, f.path, encodeToken)}`,
            ]),
        ].join('\n'));
    })
        .get('/playlist', async (req, res) => {
        const { torrent, ...params } = parseParams(req.query);
        return res.redirect(`/playlist/${encodeURIComponent(torrent)}?${querystring_1.stringify(params)}`, 301);
    });
}
exports.getStreamRouter = getStreamRouter;
